/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package application;

import globals.Globals;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import core.LevelCollectionInfo;
import core.LevelInfo;


/**
 * The dialog which offers information about the levels. 
 */
class AboutLevelsDialog extends ApplicationDialog implements ActionListener {
	JPanel content_pane;
	JPanel button_bar;

	/**
	 * Constructor for an AboutLevelsDialog which offers information corresponding to
	 * a LevelCollectionInfo.
	 * @param lci the LevelCollectionInfo object containing the information to be displayed.
	 */
	AboutLevelsDialog(LevelCollectionInfo lci) {
		super(Globals.graphics_frame, Globals.text
				.getString("about_levels_title"), true);
		content_pane = new JPanel();
		setContentPane(content_pane);
		content_pane.setOpaque(true);
		content_pane.setLayout(new BoxLayout(content_pane, BoxLayout.Y_AXIS));

		// Title
		if (lci.get_title() != null)
			addTextLine(Globals.text.getString("about_levels_name")
					+ lci.get_title());
		// Copyright
		if (lci.get_copyright() != null)
			addTextLine(Globals.text.getString("about_levels_copyright")
					+ lci.get_copyright());

		// E-Mail
		if (lci.get_email() != null)
			addTextLine(Globals.text.getString("about_levels_eMail")
					+ lci.get_email());

		// URL
		if (lci.get_url() != null)
			addTextLine(Globals.text.getString("about_levels_Url")
					+ lci.get_url());

		String[] desc = lci.get_description();
		// Description
		if (desc != null && desc.length > 0) {
			addTextLine(Globals.text.getString("about_levels_description"));
			for (int j = 0; j < desc.length; j++)
				addTextLine("    " + desc[j]);
		}

		// Level Entries
		// LevelEntry[] lea = lli.get_level_entries();

		String[] lea = process_level_info(lci);

		if (lea.length > 0) {
			addTextLine(Globals.text.getString("about_levels_entries"));
			// FIXME Is there a quick way to disable selections on a JList
			JList list = new JList(lea);
			JPanel panel = new JPanel();
			panel.add(new JScrollPane(list));
			content_pane.add(panel);
			list.setVisibleRowCount(StrictMath.min(5, lea.length));
			list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}

		// BUTTON BAR
		button_bar = new JPanel();
		JButton okay_button = new JButton(Globals.text.getString("okay_button"));
		okay_button.addActionListener(this);
		button_bar.add(okay_button);

		content_pane.add(button_bar);

		pack();
		setVisible(true);
	}

	/**
	 * Turn a levelCollectionInfo into an array of printable strings.
	 */
	private String[] process_level_info(LevelCollectionInfo lci) {
		LinkedList<String> sl = new LinkedList<String>();
		String s;
		for (int i = 0; i < lci.size(); i++) {
			s = level_info_to_string(i + 1, lci.get(i));
			if (s != null)
				sl.add(s);
		}
		return sl.toArray(new String[0]);
	}

	/**
	 * @param num
	 *            the level number
	 * @param li
	 * @return a non-null string if there is any useful info
	 */
	private String level_info_to_string(int num, LevelInfo li) {
		String s = new String();
		if ((li.get_title() != null) && (!li.get_title().trim().equals("")))
			s = s + Globals.text.getString("about_levels_entry1")
					+ li.get_title()
					+ Globals.text.getString("about_levels_entry2");
		if ((li.get_copyright() != null)
				&& (!li.get_copyright().trim().equals("")))
			s = s + Globals.text.getString("about_levels_entry3")
					+ li.get_copyright()
					+ Globals.text.getString("about_levels_entry4");
		if (!s.equals(""))
			return Globals.text.getString("about_levels_entry0") + num + s;
		else
			return null;
	}

	private void addTextLine(String t) {
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel.add(new JLabel(t));
		content_pane.add(panel);
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command == Globals.text.getString("okay_button")) {
			cancel();
		}
	}

	void cancel() {
		setVisible(false);
		dispose();
	}
}
