/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package application;

import globals.Globals;

import java.awt.CardLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;

import core.AvailableGameManager;
import core.GameInfo;

/**
 * The dialog which offers the user instructions for the supported games.
 */
class InstructionsDialog extends ApplicationDialog implements ActionListener, ItemListener {
	String game_title;
	JPanel content_pane;
	JPanel button_bar;
	JComboBox game_chooser;
	CardLayout card_layout;
	JPanel card_panel;
	

	InstructionsDialog() {
		super(Globals.graphics_frame, Globals.text
				.getString("help_games_title"), false);
		content_pane = new JPanel();
		setContentPane(content_pane);
		content_pane.setOpaque(true);
		content_pane.setLayout(new BoxLayout(content_pane, BoxLayout.Y_AXIS));

		GameInfo[] gi = AvailableGameManager.get_gameInfos();
		String[] gn = new String[gi.length];
		for (int i = 0; i < gi.length; i++) {
			gn[i] = gi[i].get_title();
		}
		
		JPanel chooserPanel = new JPanel();
		JComboBox cb = new JComboBox(gn);
		cb.addItemListener(this);
		chooserPanel.add(cb);
		content_pane.add(chooserPanel);
		
		card_layout = new CardLayout();
		card_panel = new JPanel(card_layout);
		JComponent gamePanel;
		
		// INSTRUCTIONS
		for (GameInfo g : AvailableGameManager.get_gameInfos()) {
			gamePanel = new Box(BoxLayout.Y_AXIS);
			addAboutGame(gamePanel, g);
			addInstructions(gamePanel, g);
			card_panel.add(gamePanel, g.get_title());
		}
		
		content_pane.add(card_panel);

		// BUTTON BAR
		button_bar = new JPanel();
		JButton okay_button = new JButton(Globals.text.getString("okay_button"));
		okay_button.addActionListener(this);
		button_bar.add(okay_button);

		content_pane.add(button_bar);

		pack();
		setVisible(true);
	}

	void addAboutGame(JComponent c, GameInfo gi) {
		JPanel panel = new JPanel();
		//panel.add(new JLabel(gi.get_title()));
		panel.add(new JLabel(Globals.text.getString("original_game_design")
				+ " " + gi.get_originalGameDesigner() + ", "
				+ gi.get_originalGameYear()));
		/*
		panel.add(new JLabel(Globals.text.getString("implementation_copyright")
				+ " " + gi.get_implementationCopyrightYear() + " "
				+ gi.get_implementor()));
		panel.add(new JLabel(Globals.text.getString("comments_to")
				+ gi.get_implementorEMail()));
		*/
		c.add(panel);
	}

	void addInstructions(JComponent c, GameInfo gi) {
		Box panel = new Box(BoxLayout.PAGE_AXIS);
		panel.setBorder(new TitledBorder(new LineBorder(Color.BLACK),Globals.text.getString("instructions_section")));
		JPanel p;
		String[] desc = gi.get_gameDescription();
		for (int i = 0; i < desc.length; i++) {
			p = new JPanel(new FlowLayout(FlowLayout.LEFT));
			p.add(new JLabel(desc[i]));
			panel.add(p);
		}
		c.add(panel);
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command == Globals.text.getString("okay_button")) {
			cancel();
		}
	}
	
	public void itemStateChanged(ItemEvent evt) {
		card_layout.show(card_panel, (String)evt.getItem());
	}

	void cancel() {
		setVisible(false);
	}
}
