/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package application;

import file.LevelFileParseException;
import globals.Constants;
import globals.Globals;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Paint;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;

/**
 * Holds static routines for painting words etc to the screen
 */
public class NonlevelGraphics {
	/**
	 * Draw the title in big letters as large as possible without distorting
	 * more than allowed
	 */
	private static void bigText(String title, Paint paint, Graphics2D g2,
			Insets i, Dimension d) {
		double area_width = d.getWidth() - i.left - i.right;
		double area_height = d.getHeight() - i.top - i.bottom;

		TextLayout text = new TextLayout(title, new Font(
				Constants.font_bigFontname, 1, 96), new FontRenderContext(null,
				false, false));

		// Measure the text and the space in which to place it
		double text_width = text.getVisibleAdvance();
		double text_descent = text.getDescent();
		double text_height = text.getAscent() + text_descent;

		double x_stretch = area_width / text_width;
		double y_stretch = area_height / text_height;

		double x_offset = i.left;
		double y_offset = i.top + area_height - (text_descent * y_stretch);

		if (x_stretch * Constants.font_Xproportion > y_stretch) {
			// we shouldn't stretch so much in this direction
			// so adjust the stretch to the maximum acceptable value
			x_stretch = y_stretch / Constants.font_Xproportion;
			x_offset = x_offset + (area_width - (text_width * x_stretch)) / 2.0;
		} else if (y_stretch * Constants.font_Yproportion > x_stretch) {
			y_stretch = x_stretch / Constants.font_Yproportion;
			y_offset = y_offset - (area_height - (text_height * y_stretch))
					/ 2.0;
		}

		AffineTransform aft = new AffineTransform();

		aft.translate(x_offset, y_offset);
		aft.scale(x_stretch, y_stretch);

		g2.setPaint(paint);
		g2.fill(text.getOutline(aft));
		g2.setPaint(Color.WHITE);
		g2.draw(text.getOutline(aft));
	}

	public static void welcomeScreen(Graphics2D g2, Insets i, Dimension d) {
		double area_width = d.getWidth() - i.left - i.right;
		double area_height = d.getHeight() - i.top - i.bottom;

		TextLayout text1 = new TextLayout(Constants.text_welcome1, new Font(
				Constants.font_bigFontname, 1, 96), new FontRenderContext(null,
				false, false));
		TextLayout text2 = new TextLayout(Constants.text_welcome2, new Font(
				Constants.font_bigFontname, 1, 96), new FontRenderContext(null,
				false, false));
		TextLayout text3 = new TextLayout(Constants.text_welcome3, new Font(
				Constants.font_bigFontname, 1, 96), new FontRenderContext(null,
				false, false));

		// Measure the text
		double text_width1 = text1.getVisibleAdvance();
		double text_width2 = text2.getVisibleAdvance();
		double text_width3 = text3.getVisibleAdvance();
		double text_width = StrictMath.max(text_width1, StrictMath.max(
				text_width2, text_width3));

		double text_descent = text1.getDescent();
		double text_height1 = text1.getAscent() + text_descent;
		double text_height = text_height1 * 3;

		double x_stretch = area_width / text_width;
		double y_stretch = area_height / text_height;

		double x_offset = i.left;
		double y_offset = i.top + area_height - text_descent * y_stretch;

		if (x_stretch * Constants.font_Xproportion > y_stretch) {
			// we shouldn't stretch so much in this direction
			// so adjust the stretch to the maximum acceptable value
			x_stretch = y_stretch / Constants.font_Xproportion;
			x_offset = x_offset + (area_width - (text_width * x_stretch)) / 2.0;
		} else if (y_stretch * Constants.font_Yproportion > x_stretch) {
			y_stretch = x_stretch / Constants.font_Yproportion;
			y_offset = y_offset - (area_height - (text_height * y_stretch))
					/ 2.0;
		}

		double text_x_offset1 = x_offset
				+ ((text_width1 == text_width) ? 0
						: ((text_width - text_width1) / 2.0) * x_stretch);
		double text_x_offset2 = x_offset
				+ ((text_width2 == text_width) ? 0
						: ((text_width - text_width2) / 2.0) * x_stretch);
		double text_x_offset3 = x_offset
				+ ((text_width3 == text_width) ? 0
						: ((text_width - text_width3) / 2.0) * x_stretch);

		double text_y_offset3 = y_offset;
		double text_y_offset2 = y_offset - text_height1 * y_stretch;
		double text_y_offset1 = y_offset - text_height1 * y_stretch * 2.0;

		AffineTransform aft = new AffineTransform();
		aft.translate(text_x_offset1, text_y_offset1);
		aft.scale(x_stretch, y_stretch);
		g2.setPaint(Constants.font_welcomePaint);
		g2.fill(text1.getOutline(aft));
		g2.setPaint(Color.WHITE);
		g2.draw(text1.getOutline(aft));

		aft = new AffineTransform();
		aft.translate(text_x_offset2, text_y_offset2);
		aft.scale(x_stretch, y_stretch);
		g2.setPaint(Constants.font_welcomePaint);
		g2.fill(text2.getOutline(aft));
		g2.setPaint(Color.WHITE);
		g2.draw(text2.getOutline(aft));

		aft = new AffineTransform();
		aft.translate(text_x_offset3, text_y_offset3);
		aft.scale(x_stretch, y_stretch);
		g2.setPaint(Constants.font_welcomePaint);
		g2.fill(text3.getOutline(aft));
		g2.setPaint(Color.WHITE);
		g2.draw(text3.getOutline(aft));
	}

	public static void levelFinished(Graphics2D g2, Insets i, Dimension d) {
		NonlevelGraphics.bigText(Globals.text.getString("level_finished"),
				Constants.font_wellDonePaint, g2, i, d);
	}

	public static void levelFailed(Graphics2D g2, Insets i, Dimension d) {
		NonlevelGraphics.bigText(Globals.text.getString("level_failed"),
				Constants.font_gameOverPaint, g2, i, d);
	}

	/**
	 * A way of displaying messages in the game panel useful for applets.
	 */
	static void infoScreen(String[] text, Graphics2D g2, Insets i,
			Dimension d) {
		int y_distance = (int) (d.getHeight() - i.top - i.bottom)
				/ (text.length * 2);
		int y_offset = i.top + (int) y_distance;
		int x_offset = (int) d.getWidth() / 20;
		for (int j = 0; j < text.length; j++) {
			g2.drawString(text[j], x_offset, y_offset + (y_distance * j * 2));
		}
	}

	public static void error_urlError(LevelFileParseException the_error,
			Graphics2D g2, Insets i, Dimension d) {
		String[] text = { Globals.text.getString("error_title"),
				the_error.getText1(), the_error.getText2(),
				Globals.text.getString("error_urlNext") };
		infoScreen(text,g2,i,d);
	}
}
