/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package core;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.util.LinkedList;
import java.util.Observer;

import tessellation.Direction;
import tessellation.Position;

/**
 * An interface for classes which store a level (an individual puzzle)
 */
public interface LevelInterface extends LevelInfo {
	/**
	 * Tries to make the given person walk in the given direction.
	 * @return true if the action changed the level. 
	 */
	public boolean attempt_direction(int p, Direction dir);
	
	/**
	 * Cycle through the selectable elements of the level
	 */
	public boolean attempt_advanceSelection(int s);

	/**
	 * Return the position corresponding to the point.
	 */
	public Position get_position(Point p);
	
	/**
	 * The game information for this level.
	 */
	public GameInfo get_gameInfo();
	
	/**
	 * Returns true if the puzzle has been completed
	 */
	public boolean is_completed();
	
	/**
	 * Returns true if the user has failed the puzzle
	 */
	public boolean is_failed();
	
	/**
	 * Note that ObservableProxies do not get copied.
	 */
	public Object clone();
		
	// FIXME should have a copyright accessor here.
	
	public void paintPanel(Graphics2D g2, Insets i, Dimension d);
	
	/**
	 * Attempt to select the shape at position pos.
	 */
	public boolean attempt_selectAt(int p, Position pos);
	
	/**
	 * Find the shortest route for the selection p to reach position pos or return null. 
	 */
	LinkedList<Direction> findRoute (int p, Position pos);
	
	/*
	 * WRAPPERS FOR THE OBSERVABLE PROXY
	 */

	void setChanged();

	void notifyObservers();

	void notifyObservers(Object o);

	public void deleteObserver(Observer o);
	public void addObserver(Observer o);
}
