/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package file;

/* LevelFileFilter.java -
 *
 * Copyright (C) 2002-2005 Malcolm Tyrrell
 * malcohol@eircom.net
 *
 * This code may be used and distributed under the terms of the GNU General
 * Public Licence.
 */

import java.io.File;
import javax.swing.filechooser.FileFilter;

/**
 * A file filter which is constructed by providing an array of Strings
 * representing extensions without the ".".
 */
public class LevelFileFilter extends FileFilter {
	private String[] the_extensions;
	private String the_description;

	LevelFileFilter(String[] extensions, String description) {
		// I'm typically very lazy about cloning arguments, but here I
		// have to do it
		the_extensions = (String[]) extensions.clone();
		the_description = description;
	}

	LevelFileFilter(String description) {
		the_extensions = new String[0];
		the_description = description;
	}

	public LevelFileFilter(String extension, String description) {
		the_extensions = new String[1];
		the_extensions[0] = extension;
		the_description = description;
	}

	/**
	 * Make this filter also accept files acceptable to the provided filter
	 */
	void addFilter(LevelFileFilter lff) {
		int i;
		String[] new_extensions = new String[the_extensions.length
				+ lff.the_extensions.length];
		for (i = 0; i < the_extensions.length; i++)
			new_extensions[i] = the_extensions[i];
		for (i = 0; i < lff.the_extensions.length; i++)
			new_extensions[the_extensions.length + i] = lff.the_extensions[i];
		the_extensions = new_extensions;
	}

	public boolean accept(File f) {
		if (f.isDirectory())
			return true;
		return accept(f.toString());
	}

	/**
	 * Is the file at the resource named filename a suitable file.
	 */
	public boolean accept(String s) {
		int i;
		for (i = 0; i < the_extensions.length; i++) {
			if ((s.length() > the_extensions[i].length())
					&& (s.lastIndexOf(the_extensions[i]) == s.length()
							- (the_extensions[i].length())))
				return true;
		}
		return false;
	}

	public String getDescription() {
		return the_description;
	}
	
	public String[] getExtensions() {
		return the_extensions;
	}
}
