/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package file;

import general.StringList;

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.LinkedList;

import level.Level;
import core.LevelCollection;

/**
 * class which holds some static routines for parsing levels from lists of
 * strings
 */
class StandardLevelFileParser extends LevelFileParser {
	protected LevelParser the_parser;

	StandardLevelFileParser(LevelParser lp) {
		the_parser = lp;
	}

	/**
	 * Does the string represent a valid level line.
	 * 
	 * Subclass must overload this method
	 */
	boolean levelLine(String s) {
		return the_parser.isLevelLine(s);
	}

	LevelFileFilter get_filter() {
		return the_parser.get_filter();
	}

	LinkedList<LevelCollection> parseLevels(String filename, InputStream is)
			throws LevelFileParseException {
		LinkedList<LevelCollection> ll = new LinkedList<LevelCollection>();
		ll.add(parseLevelList(filename, is));
		return ll;
	}

	/**
	 * Parses the string list and returns a list of levels
	 */
	LevelCollection parseLevelList(String filename, InputStream is)
			throws LevelFileParseException {
		StringList sl;
		Iterator<String> sli;
		LinkedList<String> tempList = new LinkedList<String>();
		// A LevelList parsed from a standard level file has no global details.
		LinkedList<Level> the_list = new LinkedList<Level>();
		String s;
		String author = "";
		String title = "";
		int levelDepth = 0;
		int linecount = 0, startline = 0;
		int levelNo = 0;

		// read the input stream into a list of strings
		try {
			sl = new StringList(is);
		} catch (IOException e) {
			throw new LevelFileParseException(e.getMessage());
		}

		// loop over all the strings in the list
		for (sli = sl.iterator(); sli.hasNext();) {
			// we only trim on the right because several parsers are sensitive
			// to the amount of whitespace on the left of level lines
			s = rightTrim(sli.next());
			linecount = linecount + 1;

			if (levelLine(s)) {
				if (tempList.isEmpty()) {
					startline = linecount;
				}
				tempList.add(s);
			} else {
				if (!tempList.isEmpty()) {
					try {
						the_list.add(the_parser.parseLevel(title, author,
								tempList.toArray(new String[0]), levelNo,
								levelDepth));
						levelNo = levelNo + 1;
					} catch (LevelParseException me) {
						throw new LevelFileParseException(me, startline);
					}
					tempList = new LinkedList<String>();
					title = "";
					author = "";
				}

				if ((s.length() > 9)
						&& (s.substring(0, 9).equalsIgnoreCase(";;TITLE: "))) {
					title = s.substring(9).trim();
				} else if ((s.length() > 10)
						&& (s.substring(0, 10).equalsIgnoreCase(";;AUTHOR: "))) {
					author = s.substring(10).trim();
				} else if ((s.length() > 9)
						&& (s.substring(0, 9).equalsIgnoreCase(";;DEPTH: "))) {
					levelDepth = Integer.parseInt(s.substring(9).trim());
				}
				// FIXME This is useful for a pedantic parser - it complains
				// about
				// bad lines.
				// else if ((s.length()>=1)&&(s.charAt(0)!=';'))
				// throw new
				// LevelFileParseException(ApplicationConstants.error_badLineInFile,linecount);
			}
		}
		// FIXME Ugly duplication of code --- think about this
		if (!tempList.isEmpty()) {
			try {
				the_list.add(the_parser.parseLevel(title, author, tempList
						.toArray(new String[0]), levelNo, levelDepth));
				levelNo = levelNo + 1;
			} catch (LevelParseException me) {
				throw new LevelFileParseException(me, startline);
			}
		}
		return new LevelCollection(filename, the_list);
	}

	/**
	 * Useful helper function - returns the string with extra spaces on the
	 * right removed.
	 */
	static String rightTrim(String s) {
		return s.replaceAll(" *$", "");
	}
}
