/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package file;

import globals.Globals;

import java.io.InputStream;
import java.util.LinkedList;

import core.LevelCollection;

/**
 * A LevelFileParser which parses XML files containing levels.
 * 
 * It is really a wrapper for an XmlLevelFileHandler. The provided LevelParser
 * is used to interpret the level lines.
 */
class XmlLevelFileParser extends LevelFileParser {
	private LevelParser[] the_parsers;

	XmlLevelFileParser(LevelParser[] lp) {
		the_parsers = lp;
	}

	LevelFileFilter get_filter() {
		return new LevelFileFilter("xml", ".xml"
				+ Globals.text.getString("file_filter_files"));
	}

	LinkedList<LevelCollection> parseLevels(String filename, InputStream is)
			throws LevelFileParseException {
		return (new XmlLevelFileHandler(the_parsers)).parseLevelList(is);
	}
}