/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package globals;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Paint;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

/**
 * Class for holding application constants.
 * 
 * These should probably become properties at some point loaded from a resource
 * file.
 */
public class Constants {
	public static final String resource_file = "data/resources";
	public static final String file_xml_levels_in_jar = "data/levels.xml";
	public static final String file_configFile_filename = ".mazezam";
	public static final boolean file_saveSettingsOnExit = true;

	public static final int text_dialog_keySettingsHelpRowCount = 15;

	public static final int text_dialog_keySettingsTextColumns = 7;

	public static final int text_dialog_graphicsPropMinPoint = 5;
	public static final int text_dialog_graphicsPropMaxPoint = 97;

	/*
	 * MNEMONICS - shortcut keys to buttons and menu items. 
	 * 
	 * FIXME these strictly belong in a resource file, since they are intended
	 * to match the button names.
	 */
	public static final int mnemonic_next = KeyEvent.VK_N;
	public static final int mnemonic_prev = KeyEvent.VK_P;
	public static final int mnemonic_retry = KeyEvent.VK_R;
	public static final int mnemonic_undo = KeyEvent.VK_U;

	public static final int mnemonic_fileMenu = KeyEvent.VK_F;
	public static final int mnemonic_viewMenu = KeyEvent.VK_V;
	public static final int mnemonic_toolMenu = KeyEvent.VK_T;
	public static final int mnemonic_helpMenu = KeyEvent.VK_H;

	public static final int accelerator_exitApp = KeyEvent.VK_Q;
	public static final int accelerator_openFile = KeyEvent.VK_O;
	public static final int accelerator_openDefaults = KeyEvent.VK_D;
	public static final int accelerator_grid = KeyEvent.VK_G;
	public static final int accelerator_keySettings = KeyEvent.VK_K;
	public static final int accelerator_preferences = KeyEvent.VK_P;
	public static final int accelerator_helpApp = KeyEvent.VK_H;
	public static final int accelerator_helpGame = KeyEvent.VK_I;
	public static final int accelerator_aboutApp = KeyEvent.VK_A;
	public static final int accelerator_aboutGame = KeyEvent.VK_A;
	public static final int accelerator_aboutLevels = KeyEvent.VK_L;

	public static final int accelerator_maskApp = ActionEvent.CTRL_MASK;
	public static final int accelerator_maskGame = ActionEvent.ALT_MASK;

	/*
	 * I don't think the name of the application should be language dependent
	 */
	public static final String text_applicationTitle = "The MazezaM Engine";
	public static final String text_welcome1 = "The";
	public static final String text_welcome2 = "MazezaM";
	public static final String text_welcome3 = "Engine";

	public static final String text_applicationVersion = "Version 1.2";
	public static final String text_applicationCopyright = "\u00A9 2002-2008 Malcolm Tyrrell";
	public static final String text_applicationAuthorName = "Malcolm Tyrrell";
	public static final String text_applicationAuthorEmail = "malcoh0l@yahoo.ie";

	/**
	 * The maximum number of info texts
	 */
	public static final int infoText_number = 4;
	
	/**
	 * The minimum size the application window should occupy
	 */
	public static final Dimension graphics_minSize = new Dimension(300, 330);

	/**
	 * The preferred size the application window should occupy
	 */
	public static final Dimension graphics_prefSize = new Dimension(440, 460);

	/**
	 * The insets of the initial border
	 */
	public static final int graphics_insets = 6;
	
	/**
	 * The font used for the big text.
	 */
	public static final String font_bigFontname = "Serif";

	/**
	 * The permissable stretchiness of the big text. 0.0 = completely stretchy,
	 * 1.0 = not stretchy
	 */
	public static final double font_Xproportion = 0.7;
	public static final double font_Yproportion = 0.3;

	/**
	 * The fonts used to display big text
	 */
	public static final Paint font_welcomePaint = Color.BLUE;
	public static final Paint font_gameOverPaint = new Color(255,0,0,150);
	public static final Paint font_wellDonePaint = new Color(0,255,0,150);

	/**
	 * Names of global parameters for use in config files
	 */
	public static final String param_graphics_Xproportion = "X_stretchiness";
	public static final String param_graphics_Yproportion = "Y_stretchiness";
	public static final String param_graphics_antialiasing = "smooth_graphics";
	public static final String param_file_reloadLevelsOnReset = "reload_levels_every_game";
	public static final String param_app_queryForQuits = "ask_before_abandon_game";
	public static final String param_app_startUpGame = "start_up_game";
	public static final String param_app_lastFileLoaded = "last_file_loaded";
	
	// FIXME need to think about how to internationalise the help files
	public static final String[][] text_dialog_helpTabFile = {
			{ "Getting started", "data/gettingStarted.html" },
			{ "The menu", "data/menu.html" },
			{ "The toolbar", "data/toolbar.html" },
			{ "License/warranty", "data/licenseWarranty.html" } };

	public static final Dimension help_minSize = graphics_minSize;
	public static final Dimension help_prefSize = new Dimension(470, 470);
}
