/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package globals;

import java.io.File;
import java.io.IOException;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

import javax.swing.JApplet;
import javax.swing.JFrame;

/**
 * Holds values I want to be globally accessible, but are not settings.
 */
public class Globals {
	/**
	 * The name of the config file
	 */
	public static File file_configFile;

	public static Settings settings;

	/**
	 * Whether to save settings on exit
	 */
	public static boolean file_saveSettingsOnExit;

	/**
	 * The frame which contains the application, which is needed for creating
	 * dialogs, etc.
	 */
	public static JFrame graphics_frame;

	/**
	 * The ResourceBundle containing the text for buttons and menu items.
	 */
	public static ResourceBundle text;

	/**
	 * Set up the globals to their initial values
	 */
	public static void initialise_application() {
		/*
		 * Accessing the user's home directory may throw a SecurityException
		 * which we can ignore
		 */

		try {
			file_configFile = new File(System.getProperty("user.home", ""),
					Constants.file_configFile_filename);
		} catch (SecurityException e) {
			System.out.println("Config file failed to load");
		}

		try {
			settings = new PropertiesSettings();

			// The resource bundle contains the text for buttons etc
			text = new PropertyResourceBundle(new Globals().getClass().getClassLoader().getResourceAsStream(
							Constants.resource_file));
			//text = new PropertyResourceBundle(ClassLoader
			//		.getSystemClassLoader().getResourceAsStream(
			//				Constants.resource_file));
		} catch (IOException e) {
			// This should cause the application to exit.
			throw new Error("defaults or resource failed to load");
		}

		file_saveSettingsOnExit = Constants.file_saveSettingsOnExit;
	}
	
	/**
	 * Set up the globals to their initial values
	 */
	public static void initialise_applet(JApplet a) {
		try {
			settings = new AppletSettings(a);

			// The resource bundle contains the text for buttons etc
			text = new PropertyResourceBundle(new Globals().getClass().getClassLoader().getResourceAsStream(
							Constants.resource_file));
		} catch (IOException e) {
			// This should cause the application to exit.
			throw new Error("resource failed to load");
		}
		file_saveSettingsOnExit = false;
	}
}