/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package level;

import java.util.Iterator;
import java.util.LinkedList;

import tessellation.Position;


/**
 * A group of floors
 */
public class FloorGroup extends Floor {
	FloorGroup() {
		floorpos_list = new LinkedList<FloorPosPair>();
	}

	public char at(Position pos) {
		Iterator<FloorPosPair> fpi;
		FloorPosPair fp;
		char u = Unit.empty;
		for (fpi = floorpos_list.iterator(); fpi.hasNext();) {
			fp = fpi.next();
			u = Unit.overlap(u, fp.floor.at(pos.minus(fp.pos)));
		}
		return u;
	}

	LinkedList<Position> nonEmptyPositions() {
		return nonEmptyPositions_SPL(floorpos_list);
	}

	/**
	 * returns a list of all nonempty positions in the provided fpl
	 */
	protected LinkedList<Position> nonEmptyPositions_SPL(
			LinkedList<FloorPosPair> fpl) {
		LinkedList<Position> l = new LinkedList<Position>();
		Iterator<FloorPosPair> fpi;
		FloorPosPair fp;
		Iterator<Position> pi;
		for (fpi = fpl.iterator(); fpi.hasNext();) {
			fp = fpi.next();
			for (pi = fp.floor.nonEmptyPositions().iterator(); pi.hasNext();) {
				l.add(fp.pos.plus(pi.next()));
			}
		}
		return l;
	}

	/**
	 * returns the (floor,position) of the floor which owns the object at the
	 * position or null if no floor does.
	 */
	protected FloorPosPair owner(Position pos) {
		Iterator<FloorPosPair> fpi;
		FloorPosPair fp;
		for (fpi = floorpos_list.iterator(); fpi.hasNext();) {
			fp = fpi.next();
			if (!fp.floor.empty(pos.minus(fp.pos)))
				return fp;
		}
		return null;
	}

	public void add(FloorPosPair fp) {
		floorpos_list.add(fp);
	}

	void paintInto(UnitGraphics g, Position rel, int superSelection) {
		Iterator<FloorPosPair> fpi;
		FloorPosPair fp;
		for (fpi = floorpos_list.iterator(); fpi.hasNext();) {
			fp = fpi.next();
			fp.floor.paintInto(g, fp.pos.plus(rel), superSelection);
		}
	}

	public Object clone() {
		FloorGroup new_instance = (FloorGroup) super.clone();
		LinkedList<FloorPosPair> new_list = new LinkedList<FloorPosPair>();
		Iterator<FloorPosPair> fpli;
		for (fpli = floorpos_list.iterator(); fpli.hasNext();) {
			new_list.add((FloorPosPair) fpli.next().clone());
		}
		new_instance.floorpos_list = new_list;
		return new_instance;
	}

	/**
	 * The list of floors and their positions
	 */
	LinkedList<FloorPosPair> floorpos_list;
}
