/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package sokoban;

import java.util.Iterator;
import java.util.LinkedList;

import level.Crate;
import level.Floor;
import level.FloorPosPair;
import level.Goal;
import level.Level;
import level.Unit;
import level.UnitShapeGroup;
import level.UnitShapePosPair;
import tessellation.Direction;
import tessellation.Position;
import tessellation.TessellationManager;
import core.GameInfo;


/**
 * Class which stores a single oban level.
 */
class ObanLevel extends Level {
	static final int width_border = 1;
	static final int height_border = 1;
	private int goals_exposed;
	private GameInfo game_info;
	private boolean is_multiban;
	
	/**
	 * Constructs an oban level.
	 */
	ObanLevel(String t, String c, LinkedList<Position> start_loc,
			LinkedList<Position> crates, LinkedList<Position> goals,
			UnitShapeGroup walls, Floor f, GameInfo ts) {
		super(t, c, start_loc);
		Iterator<Position> pli;
		Iterator<UnitShapePosPair> uspli;
		// UnitShapePosPair usp;
		title = t;
		copyright = c;
		goals_exposed = 0;
		game_info = ts;
		is_multiban = start_loc.size() > 1;

		for (pli = crates.iterator(); pli.hasNext();) {
			unitshape_group.add(new UnitShapePosPair(new Crate(), pli.next()));
		}

		for (pli = goals.iterator(); pli.hasNext();) {
			Position p = pli.next();
			floor_group.add(new FloorPosPair(new Goal(), p));
			if (!crates.contains(p))
				goals_exposed = goals_exposed + 1;
		}

		// FIXME Shouldn't be able to access walls.shapepos_list directly
		for (uspli = walls.shapepos_list.iterator(); uspli.hasNext();) {
			unitshape_group.add(uspli.next());
		}

		floor_group.add(new FloorPosPair(f, TessellationManager.newPosition()));
	}

	public boolean is_completed() {
		return goals_exposed == 0;
	}

	public void floorMove(Direction dir, Position pos) {
		// adjust goals_exposed
		if ((unitshape_group.at(pos) == Unit.block)
				&& (floor_group.at(pos.neighbour(dir)) == Unit.goal)) {
			goals_exposed = goals_exposed - 1;
		}
		if ((unitshape_group.at(pos) == Unit.block)
				&& (floor_group.at(pos) == Unit.goal)) {
			goals_exposed = goals_exposed + 1;
		}
		;
	}
	
	public GameInfo get_gameInfo () {
		if (is_multiban) {
			return new GameInfo (game_info.get_title() + " / Multiban", game_info.get_originalGameDesigner(), game_info.get_originalGameYear(), game_info.get_implementor(), game_info.get_implementationCopyrightYear(), game_info.get_implementorEMail(), game_info.get_gameDescription(), game_info.get_tessellation());	
		} else
			return game_info;
			
	}
}
