/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package tessellation;

/**
 * A direction in the Square Tessellation.
 */
public class SquareDirection extends Direction {
	public final static int north = 0;
	public final static int east = 1;
	public final static int south = 2;
	public final static int west = 3;

	final static int squareNumberOfDirections = 4;

	/**
	 * the default constructor returns a direction which is likely to occur on
	 * the outside earlier than on the inside when looking for an exposed edge
	 * in a chunk (eg north but not south).
	 */
	SquareDirection() {
		dir = north;
		numberOfDirections = squareNumberOfDirections;
	}

	/**
	 * the startDir should be one of the direction constants
	 */
	SquareDirection(int startDir) {
		numberOfDirections = squareNumberOfDirections;
		if ((startDir < 0) || (startDir >= numberOfDirections))
			throw new Error("No such direction as " + startDir);
		dir = startDir;
	}
}